@@ -15,9 +15,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: generic.c,v 1.30 2008-07-30 09:15:51 hanke Exp $
  */
@@ -62,7 +60,9 @@ static void generic_child_close(TModuleDoublePipe dpipe);
 
 void generic_set_rate(signed int rate);
 void generic_set_pitch(signed int pitch);
+void generic_set_pitch_range(signed int pitch_range);
 void generic_set_voice(SPDVoiceType voice);
+void generic_set_synthesis_voice(char *name);
 void generic_set_language(char *language);
 void generic_set_volume(signed int volume);
 void generic_set_punct(SPDPunctuation punct);
@@ -70,10 +70,13 @@ void generic_set_punct(SPDPunctuation punct);
 /* Fill the module_info structure with pointers to this modules functions */
 
 MOD_OPTION_1_STR(GenericExecuteSynth)
+    MOD_OPTION_1_STR(GenericCmdDependency)
+
     MOD_OPTION_1_INT(GenericMaxChunkLength)
     MOD_OPTION_1_STR(GenericDelimiters)
     MOD_OPTION_1_STR(GenericPunctNone)
     MOD_OPTION_1_STR(GenericPunctSome)
+    MOD_OPTION_1_STR(GenericPunctMost)
     MOD_OPTION_1_STR(GenericPunctAll)
     MOD_OPTION_1_STR(GenericStripPunctChars)
     MOD_OPTION_1_STR(GenericRecodeFallback)
@@ -84,12 +87,16 @@ MOD_OPTION_1_STR(GenericExecuteSynth)
     MOD_OPTION_1_INT(GenericPitchAdd)
     MOD_OPTION_1_FLOAT(GenericPitchMultiply)
     MOD_OPTION_1_INT(GenericPitchForceInteger)
+    MOD_OPTION_1_INT(GenericPitchRangeAdd)
+    MOD_OPTION_1_FLOAT(GenericPitchRangeMultiply)
+    MOD_OPTION_1_INT(GenericPitchRangeForceInteger)
     MOD_OPTION_1_INT(GenericVolumeAdd)
     MOD_OPTION_1_FLOAT(GenericVolumeMultiply)
     MOD_OPTION_1_INT(GenericVolumeForceInteger)
     MOD_OPTION_3_HT(GenericLanguage, code, name, charset)
 
 static char generic_msg_pitch_str[16];
+static char generic_msg_pitch_range_str[16];
 static char generic_msg_rate_str[16];
 static char generic_msg_volume_str[16];
 static char *generic_msg_voice_str = NULL;
@@ -103,6 +110,7 @@ int module_load(void)
 	INIT_SETTINGS_TABLES();
 
 	MOD_OPTION_1_STR_REG(GenericExecuteSynth, "");
+	MOD_OPTION_1_STR_REG(GenericCmdDependency, "");
 
 	REGISTER_DEBUG();
 
@@ -119,16 +127,22 @@ int module_load(void)
 	MOD_OPTION_1_FLOAT_REG(GenericPitchMultiply, 1);
 	MOD_OPTION_1_INT_REG(GenericPitchForceInteger, 0);
 
+	MOD_OPTION_1_INT_REG(GenericPitchRangeAdd, 0);
+	MOD_OPTION_1_FLOAT_REG(GenericPitchRangeMultiply, 1);
+	MOD_OPTION_1_INT_REG(GenericPitchRangeForceInteger, 0);
+
 	MOD_OPTION_1_INT_REG(GenericVolumeAdd, 0);
 	MOD_OPTION_1_FLOAT_REG(GenericVolumeMultiply, 1);
 	MOD_OPTION_1_INT_REG(GenericVolumeForceInteger, 0);
 
 	MOD_OPTION_HT_REG(GenericLanguage);
 
-	MOD_OPTION_1_STR_REG(GenericPunctNone, "--punct-none");
-	MOD_OPTION_1_STR_REG(GenericPunctSome, "--punct-some");
-	MOD_OPTION_1_STR_REG(GenericPunctAll, "--punct-all");
+	MOD_OPTION_1_STR_REG(GenericPunctNone, "");
+	MOD_OPTION_1_STR_REG(GenericPunctSome, "");
+	MOD_OPTION_1_STR_REG(GenericPunctMost, "");
+	MOD_OPTION_1_STR_REG(GenericPunctAll, "");
 
+	module_register_available_voices();
 	module_register_settings_voices();
 
 	return 0;
@@ -143,10 +157,11 @@ int module_init(char **status_info)
 	DBG("GenericMaxChunkLength = %d\n", GenericMaxChunkLength);
 	DBG("GenericDelimiters = %s\n", GenericDelimiters);
 	DBG("GenericExecuteSynth = %s\n", GenericExecuteSynth);
+	DBG("GenericCmdDependency = %s\n", GenericCmdDependency);
 
 	generic_msg_language =
 	    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
-	generic_msg_language->code = g_strdup("en");
+	generic_msg_language->code = g_strdup("en-US");
 	generic_msg_language->charset = g_strdup("iso-8859-1");
 	generic_msg_language->name = g_strdup("english");
 
@@ -172,7 +187,7 @@ int module_init(char **status_info)
 
 SPDVoice **module_list_voices(void)
 {
-	return NULL;
+	return module_list_registered_voices();
 }
 
 int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
@@ -185,11 +200,12 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 		DBG("Speaking when requested to write");
 		return 0;
 	}
-
+	UPDATE_STRING_PARAMETER(voice.name, generic_set_synthesis_voice);
 	UPDATE_STRING_PARAMETER(voice.language, generic_set_language);
 	UPDATE_PARAMETER(voice_type, generic_set_voice);
 	UPDATE_PARAMETER(punctuation_mode, generic_set_punct);
 	UPDATE_PARAMETER(pitch, generic_set_pitch);
+	UPDATE_PARAMETER(pitch_range, generic_set_pitch_range);
 	UPDATE_PARAMETER(rate, generic_set_rate);
 	UPDATE_PARAMETER(volume, generic_set_volume);
 
@@ -216,6 +232,7 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	if (tmp == NULL)
 		return -1;
 
+	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	if (msgtype == SPD_MSGTYPE_TEXT)
 		*generic_message = module_strip_ssml(tmp);
 	else
@@ -418,6 +435,9 @@ void *_generic_speak(void *nothing)
 				e_string =
 				    string_replace(e_string, "$PITCH",
 						   generic_msg_pitch_str);
+				e_string =
+				    string_replace(e_string, "$PITCH_RANGE",
+						   generic_msg_pitch_range_str);
 				e_string =
 				    string_replace(e_string, "$RATE",
 						   generic_msg_rate_str);
@@ -580,6 +600,21 @@ void generic_set_pitch(int pitch)
 	}
 }
 
+void generic_set_pitch_range(int pitch_range)
+{
+	float hpitch_range;
+
+	hpitch_range =
+	    ((float)pitch_range) * GenericPitchRangeMultiply +
+	    GenericPitchRangeAdd;
+	if (!GenericPitchRangeForceInteger) {
+		snprintf(generic_msg_pitch_range_str, 15, "%.2f", hpitch_range);
+	} else {
+		snprintf(generic_msg_pitch_range_str, 15, "%d",
+			 (int)hpitch_range);
+	}
+}
+
 void generic_set_rate(int rate)
 {
 	float hrate;
@@ -609,9 +644,17 @@ void generic_set_volume(int volume)
 
 void generic_set_language(char *lang)
 {
-
+	DBG("Setting language %s", lang);
+	char *dash = strchr(lang, '-');
 	generic_msg_language =
 	    (TGenericLanguage *) get_ht_option(GenericLanguage, lang);
+	if (generic_msg_language == NULL && dash) {
+		char *lang_only = g_strdup(lang);
+		lang_only[dash-lang] = 0;
+		generic_msg_language =
+		    (TGenericLanguage *) get_ht_option(GenericLanguage, lang_only);
+		g_free(lang_only);
+	}
 	if (generic_msg_language == NULL) {
 		DBG("Language %s not found in the configuration file, using defaults.", lang);
 		generic_msg_language =
@@ -626,7 +669,7 @@ void generic_set_language(char *lang)
 		    lang);
 		generic_msg_language =
 		    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
-		generic_msg_language->code = g_strdup("en");
+		generic_msg_language->code = g_strdup("en-US");
 		generic_msg_language->charset = g_strdup("iso-8859-1");
 		generic_msg_language->name = g_strdup("english");
 	}
@@ -644,6 +687,13 @@ void generic_set_voice(SPDVoiceType voice)
 	}
 }
 
+void generic_set_synthesis_voice(char *name)
+{
+	assert(msg_settings.voice.name);
+	if (module_existsvoice(msg_settings.voice.name))
+		generic_msg_voice_str = msg_settings.voice.name;
+}
+
 void generic_set_punct(SPDPunctuation punct)
 {
 	if (punct == SPD_PUNCT_NONE) {
@@ -652,6 +702,13 @@ void generic_set_punct(SPDPunctuation punct)
 	} else if (punct == SPD_PUNCT_SOME) {
 		generic_msg_punct_str = g_strdup((char *)GenericPunctSome);
 		return;
+	} else if (punct == SPD_PUNCT_MOST) {
+		if (GenericPunctMost[0] == '\0' && GenericPunctSome[0] != '\0')
+			/* Compatibility with old configuration files */
+			generic_msg_punct_str = g_strdup((char *)GenericPunctSome);
+		else
+			generic_msg_punct_str = g_strdup((char *)GenericPunctMost);
+		return;
 	} else if (punct == SPD_PUNCT_ALL) {
 		generic_msg_punct_str = g_strdup((char *)GenericPunctAll);
 		return;
